/*  lv_error.c
 * 
 *  LabVIEW Error Cluster Utilities
 *
 *  Copyright 2004-2014 Martin Henz
 *  All rights reserved.
 *
 *	E-Mail: martin.henz@mhst.de
 *	Web: http://www.mhst.de
 *
 *
 *
 *  Redistribution and use in source and binary forms, with or without modification, 
 *  are permitted provided that the following conditions are met:
 *  
 *  * Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 *  
 *  * Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 *  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 *  SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *  OF SUCH DAMAGE.
 * */

#define _WIN32_WINNT 0x0500
#include <windows.h>
#include <crtdbg.h>

#include "lv_error.h"

/*  lv_ErrBld
 *
 *  Setzt den Fehler oder Warnungsstatus des Error Clusters
 *
 *  parameter
 *      lfStatus		neuer Fehlerstatus
 *      lCode			neuer Fehlercode
 *      pszSource		Fehlerquelle
 *      pError			ErrorCluster
 *
 */
void lv_ErrBld(LVBoolean lfStatus, long lCode, char *pszSource, pErrC pError)
{
	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	/* Der ErrorCluster wird nur verndert, wenn der aktuelle Status False ist, oder
	 * wenn der aktuelle Errorcode 0 ist (fr Warnungen)
	 */
	if ( pError->lfStatus == LVFALSE )
	{
		_ASSERT(pszSource);

	    if (lfStatus == LVTRUE || (pError->lCode == 0 && lCode != 0) )
	    {
		    pError->lfStatus = lfStatus;
		    pError->lCode = lCode;
		    LStrPrintf(pError->shSource,(CStr)"%s", pszSource);
		}
	}
}

/*  lv_ErrBldMsg
 *
 *  Setzt den Fehler oder Warnungsstatus des Error Clusters
 *
 *  parameter
 *      lfStatus		neuer Fehlerstatus
 *      lCode			neuer Fehlercode
 *      pszSource		Fehlerquelle
 *		pszMsg			Nachricht
 *      pError			ErrorCluster
 *
 */
void lv_ErrBldMsg(LVBoolean lfStatus, long lCode, char *pszSource, char *pszMsg, pErrC pError)
{
	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	/* Der ErrorCluster wird nur verndert, wenn der aktuelle Status False ist, oder
	 * wenn der aktuelle Errorcode 0 ist (fr Warnungen)
	 */
	if ( pError->lfStatus == LVFALSE )
	{
		_ASSERT(pszSource);

	    if (lfStatus == LVTRUE || (pError->lCode == 0 && lCode != 0) )
	    {
		    pError->lfStatus = lfStatus;
		    pError->lCode = lCode;
			LStrPrintf(pError->shSource,(CStr)"%s: %s", pszSource, pszMsg);
		}
	}
}

/*  lv_ErrBldOpt
 *
 *  Setzt den Fehler oder Warnungsstatus des Error Clusters
 *
 *  parameter
 *      lfStatus		neuer Fehlerstatus
 *      lCode			neuer Fehlercode
 *      pszSource		Fehlerquelle
 *		pszMsg			Nachricht
 *		pszOptMsg		Optionale Nachricht
 *      pError			ErrorCluster
 */
void lv_ErrBldOpt(LVBoolean lfStatus, long lCode, char *pszSource, char *pszMsg, char *pszOptMsg, pErrC pError)
{
	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	/* Der ErrorCluster wird nur verndert, wenn der aktuelle Status False ist, oder
	 * wenn der aktuelle Errorcode 0 ist (fr Warnungen)
	 */
	if ( pError->lfStatus == LVFALSE )
	{
		_ASSERT(pszSource);

		if (lfStatus == LVTRUE || (pError->lCode == 0 && lCode != 0) )
	    {
		    pError->lfStatus = lfStatus;
		    pError->lCode = lCode;
			if (pszMsg != NULL)
			{
				if (pszOptMsg != NULL)
					LStrPrintf(pError->shSource,(CStr)"%s: %s (%s)", pszSource, pszMsg, pszOptMsg);
				else
					LStrPrintf(pError->shSource,(CStr)"%s: %s", pszSource, pszMsg);
			}
			else
			{
				if (pszOptMsg != NULL)
					LStrPrintf(pError->shSource,(CStr)"%s (%s)", pszSource, pszOptMsg);
				else
					LStrPrintf(pError->shSource,(CStr)"%s", pszSource);
			}
		}
	}
}

/*  lv_ErrBuilder
 *
 *  Setzt den Fehler oder Warnungsstatus des Error Clusters
 *
 *  parameter
 *      lfStatus		neuer Fehlerstatus
 *      lCode			neuer Fehlercode
 *      pszSource		Fehlerquelle
 *		pszMsg			Nachricht
 *		pszOptMsg		Optionale Nachricht
 *		pszReason		mgliche Ursache
 *      pError			ErrorCluster
 */
void lv_ErrBuilder(LVBoolean lfStatus, long lCode, char *pszSource, char *pszMsg, char *pszOptMsg, char *pszReason, pErrC pError)
{
	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	/* Der ErrorCluster wird nur verndert, wenn der aktuelle Status False ist, oder
	 * wenn der aktuelle Errorcode 0 ist (fr Warnungen)
	 */
	if ( pError->lfStatus == LVFALSE )
	{
		_ASSERT(pszSource);

	    if (lfStatus == LVTRUE || (pError->lCode == 0 && lCode != 0) )
	    {
		    pError->lfStatus = lfStatus;
		    pError->lCode = lCode;
			if (pszMsg != NULL)
			{
				if (pszOptMsg != NULL)
				{
					if (pszReason != NULL)
						LStrPrintf(pError->shSource,(CStr)"%s: %s (%s)<err>%s", pszSource, pszMsg, pszOptMsg, pszReason);
					else
						LStrPrintf(pError->shSource,(CStr)"%s: %s (%s)", pszSource, pszMsg, pszOptMsg);
				}
				else
				{
					if (pszReason != NULL)
						LStrPrintf(pError->shSource,(CStr)"%s: %s<err>%s", pszSource, pszMsg, pszReason);
					else
						LStrPrintf(pError->shSource,(CStr)"%s: %s", pszSource, pszMsg);
				}
			}
			else
			{
				if (pszOptMsg != NULL)
				{
					if (pszReason != NULL)
						LStrPrintf(pError->shSource,(CStr)"%s (%s)<err>%s", pszSource, pszOptMsg, pszReason);
					else
						LStrPrintf(pError->shSource,(CStr)"%s (%s)", pszSource, pszOptMsg);
				}
				else
				{
					if (pszReason != NULL)
						LStrPrintf(pError->shSource,(CStr)"%s<err>%s", pszSource, pszReason);
					else
						LStrPrintf(pError->shSource,(CStr)"%s", pszSource);
				}
			}
		}
	}
}

/*  lv_EasyErrBld
 *
 *  Setzt den Fehlerstatus des Error Clusters
 *
 *  parameter
 *      lCode			neuer Fehlercode
 *      pszSource		Fehlerquelle
 *      pError			ErrorCluster
 */
void lv_EasyErrBld(long lCode, char *pszSource, pErrC pError)
{
	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	/* Der ErrorCluster wird nur verndert, wenn der aktuelle Status False ist, oder
	 * wenn der aktuelle Errorcode 0 ist (fr Warnungen)
	 */
	if ( pError->lfStatus == LVFALSE )
	{
		_ASSERT(pszSource);

	    if (lCode != 0)
	    {
		    pError->lfStatus = LVTRUE;
		    pError->lCode = lCode;
		    LStrPrintf(pError->shSource, (CStr)"%s", pszSource);
		}
	}
}

/*  lv_WinErrBld
 *
 *  Setzt den Windows Fehlerstatus des Error Clusters
 *
 *  input
 *      lCode			neuer Fehlercode
 *      pszSource		Fehlerquelle
 *      pError			ErrorCluster
 *
 *  output
 *      error           ErrorCluster
 */
void lv_WinErrBld(long lCode, char *pszSource, pErrC pError)
{
	char * pszWinErrMsg;

	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	/* Der ErrorCluster wird nur verndert, wenn der aktuelle Status False ist, oder
	 * wenn der aktuelle Errorcode 0 ist (fr Warnungen)
	 */
	if ( pError->lfStatus == LVFALSE )
	{
		_ASSERT(pszSource);

	    if (lCode != 0)
	    {
		    pError->lfStatus = LVTRUE;
		    pError->lCode = lCode;

			FormatMessage(
				FORMAT_MESSAGE_ALLOCATE_BUFFER | 
				FORMAT_MESSAGE_FROM_SYSTEM | 
				FORMAT_MESSAGE_IGNORE_INSERTS,
				NULL,
				lCode,
				MAKELANGID(LANG_NEUTRAL, SUBLANG_DEFAULT),
				(LPSTR) &pszWinErrMsg,
				0,
				NULL);

			if (pszWinErrMsg != NULL)
			{
				LStrPrintf(pError->shSource, (CStr)"%s<err>%s", pszSource, pszWinErrMsg);
				LocalFree(pszWinErrMsg);
			}
			else
				LStrPrintf(pError->shSource, (CStr)"%s", pszSource);
		}
	}
}

/*  lv_WarnOrErrBld
 *
 *  Setzt den Fehlerstatus des Error Clusters. The error status will be set
 *  for Error Codes lower than 60000.
 *
 *  parameter
 *      lCode			neuer Fehlercode
 *      pszSource		Fehlerquelle
 *      pError			ErrorCluster
 *
 */
void lv_WarnOrErrBld(long lCode, char *pszSource, pErrC pError)
{
	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	/* Der ErrorCluster wird nur verndert, wenn der aktuelle Status False ist, oder
	 * wenn der aktuelle Errorcode 0 ist (fr Warnungen)
	 */
	if ( pError->lfStatus == LVFALSE )
	{
		_ASSERT(pszSource);

	    if (lCode != 0)
	    {
			pError->lfStatus = lCode < 60000 ? LVTRUE : LVFALSE;
		    pError->lCode = lCode;
		    LStrPrintf(pError->shSource, (CStr)"%s", pszSource);
		}
	}
}

/*  lv_LStrErrBld
 *
 *  Setzt den Fehler oder Warnungsstatus des Error Clusters
 *
 *  parameter
 *      lfStatus		neuer Fehlerstatus
 *      lCode			neuer Fehlercode
 *      shSource		Fehlerquelle
 *      pError			ErrorCluster
 */
void lv_LStrErrBld(LVBoolean *plfStatus, long lCode, LStrHandle shSource, pErrC pError)
{
	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	/* Der ErrorCluster wird nur verndert, wenn der aktuelle Status False ist, oder
	 * wenn der aktuelle Errorcode 0 ist (fr Warnungen)
	 */
	if ( pError->lfStatus == LVFALSE )
	{
		_ASSERT(!DSCheckHandle(shSource));

	    if (*plfStatus == LVTRUE || (pError->lCode == 0 && lCode != 0))
	    {
			pError->lfStatus = *plfStatus;
	    	pError->lCode = lCode;
		    LStrPrintf(pError->shSource, (CStr)"%H", shSource);
		}
	}
}

/*  lv_LStrEasyErrBld
 *
 *  Setzt den Fehlerstatus des Error Clusters
 *
 *  parameter
 *      lCode			neuer Fehlercode
 *      shSource		Fehlerquelle
 *      pError			ErrorCluster
 */
void lv_LStrEasyErrBld(long lCode, LStrHandle shSource, pErrC pError)
{
	//error pointer test
	_ASSERT(!DSCheckPtr(pError));

	/* Der ErrorCluster wird nur verndert, wenn der aktuelle Status False ist, oder
	 * wenn der aktuelle Errorcode 0 ist (fr Warnungen)
	 */
	if ( pError->lfStatus == LVFALSE )
	{
		_ASSERT(!DSCheckHandle(shSource));

	    if ( lCode != 0 )
	    {
			pError->lfStatus = LVTRUE;
			pError->lCode = lCode;
			LStrPrintf(pError->shSource, (CStr)"%H", shSource);
		}
	}
}
