/*  plist.h
 * 
 *  Simple List Implementation
 *
 *  Copyright 2004-2014 Martin Henz
 *  All rights reserved.
 *
 *	E-Mail: martin.henz@mhst.de
 *	Web: http://www.mhst.de
 *
 *
 *  Redistribution and use in source and binary forms, with or without modification, 
 *  are permitted provided that the following conditions are met:
 *  
 *  * Redistributions of source code must retain the above copyright notice, 
 *    this list of conditions and the following disclaimer.
 *  
 *  * Redistributions in binary form must reproduce the above copyright notice, 
 *    this list of conditions and the following disclaimer in the documentation 
 *    and/or other materials provided with the distribution.
 *  
 *  THIS SOFTWARE IS PROVIDED BY THE REGENTS AND CONTRIBUTORS "AS IS" AND ANY EXPRESS 
 *  OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF 
 *  MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT 
 *  SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, 
 *  SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, 
 *  PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR 
 *  BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 *  CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING 
 *  IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY 
 *  OF SUCH DAMAGE.
 * */

#ifndef __inc_plist_h
#define __inc_plist_h

#pragma pack(1)

/*
 *  The LINK structure is used to chain structures together into a list.
 */

typedef struct _LINK *PLINK;

typedef struct _LINK {
    PLINK pPrev;					//previous or back pointer
    PLINK pNext;					//next or forward pointer
	void *pElement;					//element pointer
} LINK;

/*
 *  The LIST data structure.
 */

typedef struct _LIST {
    PLINK pTail;						//list tail pointer
    PLINK pHead;						//list head pointer
    unsigned long ulLength;				//list length
} LIST;

typedef struct _LIST *PLIST;

#pragma pack()

/*
 *  Functions
 */

#if defined(__cplusplus) || defined(__cplusplus__)
   extern "C" {
#endif

//create and dispose
PLIST ListCreate(void);
int ListDispose(PLIST pList);

//add to list
void * ListAppendElement(PLIST pList, void *pElement);

//remove from list
void * ListRemoveHandle(PLIST pList, void *hElement);
void * ListRemoveFirstElement(PLIST pList);
void * ListRemoveLastElement(PLIST pList);

//get element and element handle
void * ListGetElement(PLIST pList, void *hElement);
void * ListGetFirstElement(PLIST pList);
void * ListGetNextElement(PLIST pList, void *pElement);
void * ListGetFirstHandle(PLIST pList);
void * ListGetNextHandle(PLIST pList, void *hElement);

//information and check
int ListGetLength(PLIST pList);
int ListCheckHandle(PLIST pList, void *hElement);
int ListCheckElement(PLIST pList, void *pElement);

void * ListFindElement(PLIST pList, void *pElement, int ( __cdecl *compare ) ( const void *, const void *) );
void * ListFindElmHandle(PLIST pList, void *pElement, int ( __cdecl *compare ) ( const void *, const void *) );
void * ListFindHandle(PLIST pList, void *hElement, int ( __cdecl *compare ) ( const void *, const void *) );

#if defined(__cplusplus) || defined(__cplusplus__)
   }
#endif

#endif
